import { CommandBuilder, RunState } from '../core';
import { BaseContext, CliContext, MiniCli } from './Cli';
import type { HelpCommand } from './entries/help';
import type { VersionCommand } from './entries/version';
export declare type Meta<Context extends BaseContext> = {
    definitions: ((command: CommandBuilder<CliContext<Context>>) => void)[];
    transformers: ((state: RunState, command: Command<Context>, builder: CommandBuilder<CliContext<Context>>) => void)[];
};
/**
 * The usage of a Command.
 */
export declare type Usage = {
    /**
     * The category of the command.
     *
     * Included in the detailed usage.
     */
    category?: string;
    /**
     * The short description of the command, formatted as Markdown.
     *
     * Included in the detailed usage.
     */
    description?: string;
    /**
     * The extended details of the command, formatted as Markdown.
     *
     * Included in the detailed usage.
     */
    details?: string;
    /**
     * Examples of the command represented as an Array of tuples.
     *
     * The first element of the tuple represents the description of the example.
     *
     * The second element of the tuple represents the command of the example.
     * If present, the leading `$0` is replaced with `cli.binaryName`.
     */
    examples?: [string, string][];
};
/**
 * The definition of a Command.
 */
export declare type Definition = Usage & {
    /**
     * The path of the command, starting with `cli.binaryName`.
     */
    path: string;
    /**
     * The detailed usage of the command.
     */
    usage: string;
    /**
     * The various options registered on the command.
     */
    options: {
        definition: string;
        description?: string;
    }[];
};
/**
 * The schema used to validate the Command instance.
 *
 * The easiest way to validate it is by using the [Yup](https://github.com/jquense/yup) library.
 *
 * @example
 * yup.object().shape({
 *   a: yup.number().integer(),
 *   b: yup.number().integer(),
 * })
 */
export declare type Schema<C extends Command<any>> = {
    /**
     * A function that takes the `Command` instance as a parameter and validates it, throwing an Error if the validation fails.
     */
    validate: (object: C) => void;
};
export declare type CommandClass<Context extends BaseContext = BaseContext> = {
    new (): Command<Context>;
    resolveMeta(prototype: Command<Context>): Meta<Context>;
    schema?: Schema<any>;
    usage?: Usage;
};
export declare abstract class Command<Context extends BaseContext = BaseContext> {
    private static meta?;
    static getMeta<Context extends BaseContext>(prototype: Command<Context>): Meta<Context>;
    static resolveMeta<Context extends BaseContext>(prototype: Command<Context>): Meta<Context>;
    private static registerDefinition;
    private static registerTransformer;
    static addPath(...path: string[]): void;
    static addOption<Context extends BaseContext>(name: string, builder: (prototype: Command<Context>, propertyName: string) => void): void;
    /**
     * Wrap the specified command to be attached to the given path on the command line.
     * The first path thus attached will be considered the "main" one, and all others will be aliases.
     * @param path The command path.
     */
    static Path(...path: string[]): <Context extends BaseContext>(prototype: Command<Context>, propertyName: string) => void;
    /**
     * Register a boolean listener for the given option names. When Clipanion detects that this argument is present, the value will be set to false. The value won't be set unless the option is found, so you must remember to set it to an appropriate default value.
     * @param descriptor the option names.
     */
    static Boolean(descriptor: string, { hidden, description }?: {
        hidden?: boolean;
        description?: string;
    }): <Context extends BaseContext>(prototype: Command<Context>, propertyName: string) => void;
    /**
     * Register a boolean listener for the given option names. Each time Clipanion detects that this argument is present, the counter will be incremented. Each time the argument is negated, the counter will be reset to `0`. The counter won't be set unless the option is found, so you must remember to set it to an appropriate default value.

     * @param descriptor A comma-separated list of option names.
     */
    static Counter(descriptor: string, { hidden, description }?: {
        hidden?: boolean;
        description?: string;
    }): <Context extends BaseContext>(prototype: Command<Context>, propertyName: string) => void;
    /**
     * Register a listener that looks for an option and its followup argument. When Clipanion detects that this argument is present, the value will be set to whatever follows the option in the input. The value won't be set unless the option is found, so you must remember to set it to an appropriate default value.
     * Note that all methods affecting positional arguments are evaluated in the definition order; don't mess with it (for example sorting your properties in ascendent order might have adverse results).
     * @param descriptor The option names.
     */
    static String(descriptor: string, opts?: {
        arity?: number;
        tolerateBoolean?: boolean;
        hidden?: boolean;
        description?: string;
    }): PropertyDecorator;
    /**
     * Register a listener that looks for positional arguments. When Clipanion detects that an argument isn't an option, it will put it in this property and continue processing the rest of the command line.
     * Note that all methods affecting positional arguments are evaluated in the definition order; don't mess with it (for example sorting your properties in ascendent order might have adverse results).
     * @param descriptor Whether or not filling the positional argument is required for the command to be a valid selection.
     */
    static String(descriptor?: {
        required?: boolean;
        name?: string;
    }): PropertyDecorator;
    /**
     * Register a listener that looks for an option and its followup argument. When Clipanion detects that this argument is present, the value will be pushed into the array represented in the property.
     */
    static Array(descriptor: string, { arity, hidden, description }?: {
        arity?: number;
        hidden?: boolean;
        description?: string;
    }): <Context extends BaseContext>(prototype: Command<Context>, propertyName: string) => void;
    /**
     * Register a listener that takes all the positional arguments remaining and store them into the selected property.
     * Note that all methods affecting positional arguments are evaluated in the definition order; don't mess with it (for example sorting your properties in ascendent order might have adverse results).
     */
    static Rest(): PropertyDecorator;
    /**
     * Register a listener that takes all the positional arguments remaining and store them into the selected property.
     * Note that all methods affecting positional arguments are evaluated in the definition order; don't mess with it (for example sorting your properties in ascendent order might have adverse results).
     * @param opts.required The minimal number of arguments required for the command to be successful.
     */
    static Rest(opts: {
        required: number;
    }): PropertyDecorator;
    /**
     * Register a listener that takes all the arguments remaining (including options and such) and store them into the selected property.
     * Note that all methods affecting positional arguments are evaluated in the definition order; don't mess with it (for example sorting your properties in ascendent order might have adverse results).
     */
    static Proxy({ required }?: {
        required?: number;
    }): <Context extends BaseContext>(prototype: Command<Context>, propertyName: string) => void;
    /**
     * Defines the usage information for the given command.
     * @param usage
     */
    static Usage(usage: Usage): Usage;
    /**
     * Contains the usage information for the command. If undefined, the command will be hidden from the general listing.
     */
    static usage?: Usage;
    /**
     * Defines the schema for the given command.
     * @param schema
     */
    static Schema<C extends Command<any> = Command<BaseContext>>(schema: Schema<C>): Schema<C>;
    /**
     * The schema used to validate the Command instance.
     */
    static schema?: Schema<any>;
    /**
     * Standard command that'll get executed by `Cli#run` and `Cli#runExit`. Expected to return an exit code or nothing (which Clipanion will treat as if 0 had been returned).
     */
    abstract execute(): Promise<number | void>;
    /**
     * Standard error handler which will simply rethrow the error. Can be used to add custom logic to handle errors
     * from the command or simply return the parent class error handling.
     * @param error
     */
    catch(error: any): Promise<void>;
    validateAndExecute(): Promise<number>;
    /**
     * Predefined that will be set to true if `-h,--help` has been used, in which case `Command#execute` shouldn't be called.
     */
    help: boolean;
    /**
     * Predefined variable that will be populated with a miniature API that can be used to query Clipanion and forward commands.
     */
    cli: MiniCli<Context>;
    /**
     * Predefined variable that will be populated with the context of the application.
     */
    context: Context;
    /**
     * The path that got used to access the command being executed.
     */
    path: string[];
    /**
     * A list of useful semi-opinionated command entries that have to be registered manually.
     *
     * They cover the basic needs of most CLIs (e.g. help command, version command).
     *
     * @example
     * cli.register(Command.Entries.Help);
     * cli.register(Command.Entries.Version);
     */
    static Entries: {
        /**
         * A command that prints the usage of all commands.
         *
         * Paths: `-h`, `--help`
         */
        Help: typeof HelpCommand;
        /**
         * A command that prints the version of the binary (`cli.binaryVersion`).
         *
         * Paths: `-v`, `--version`
         */
        Version: typeof VersionCommand;
    };
}
